@@ -22,8 +22,9 @@ use crate::resources::{
         GetAvailableResponse as GetAvailableBucketsResponse, GetResponse as GetBucketResponse,
     },
     comment::{
-        GetAnnotationsResponse, GetLabellingsAfter, GetRecentRequest, PutCommentsRequest,
-        PutCommentsResponse, RecentCommentsPage, SyncCommentsRequest, UpdateAnnotationsRequest,
+        GetAnnotationsResponse, GetCommentResponse, GetLabellingsAfter, GetRecentRequest,
+        PutCommentsRequest, PutCommentsResponse, RecentCommentsPage, SyncCommentsRequest,
+        UpdateAnnotationsRequest,
     },
     dataset::{
         CreateRequest as CreateDatasetRequest, CreateResponse as CreateDatasetResponse,
@@ -60,15 +61,20 @@ pub use crate::{
         },
         comment::{
             AnnotatedComment, Comment, CommentFilter, CommentsIterPage, Continuation, Entity,
-            Id as CommentId, Label, LabelName, Message, MessageBody, MessageSignature,
-            MessageSubject, NewAnnotatedComment, NewComment, NewEntities, NewLabelling,
-            PropertyMap, PropertyValue, Sentiment, SyncCommentsResponse, Uid as CommentUid,
+            Id as CommentId, Label, Message, MessageBody, MessageSignature, MessageSubject,
+            NewAnnotatedComment, NewComment, NewEntities, NewLabelling, PropertyMap, PropertyValue,
+            Sentiment, SyncCommentsResponse, Uid as CommentUid,
         },
         dataset::{
             Dataset, FullName as DatasetFullName, Id as DatasetId, Identifier as DatasetIdentifier,
             Name as DatasetName, NewDataset, UpdateDataset,
         },
         email::{Id as EmailId, Mailbox, MimeContent, NewEmail},
+        entity_def::{EntityDef, Id as EntityDefId, Name as EntityName, NewEntityDef},
+        label_def::{
+            LabelDef, LabelDefPretrained, Name as LabelName, NewLabelDef, NewLabelDefPretrained,
+            PretrainedId as LabelDefPretrainedId,
+        },
         source::{
             FullName as SourceFullName, Id as SourceId, Identifier as SourceIdentifier,
             Name as SourceName, NewSource, Source, UpdateSource,
@@ -278,6 +284,19 @@ impl Client {
         CommentsIter::new(self, source_name, page_size, timerange)
     }
 
+    /// Get a single comment by id.
+    pub fn get_comment<'a>(
+        &'a self,
+        source_name: &'a SourceFullName,
+        comment_id: &'a CommentId,
+    ) -> Result<Comment> {
+        Ok(self
+            .get::<_, GetCommentResponse, SimpleApiError>(
+                self.endpoints.comment_by_id(source_name, comment_id)?,
+            )?
+            .comment)
+    }
+
     pub fn put_comments(
         &self,
         source_name: &SourceFullName,
@@ -1071,6 +1090,21 @@ impl Endpoints {
             })
     }
 
+    fn comment_by_id(&self, source_name: &SourceFullName, comment_id: &CommentId) -> Result<Url> {
+        self.base
+            .join(&format!(
+                "/api/v1/sources/{}/comments/{}",
+                source_name.0, comment_id.0
+            ))
+            .map_err(|source| Error::UrlParseError {
+                source,
+                message: format!(
+                    "Could not build get comment by id URL for source `{}`, comment `{}`.",
+                    source_name.0, comment_id.0,
+                ),
+            })
+    }
+
     fn comments_v1(&self, source_name: &SourceFullName) -> Result<Url> {
         self.base
             .join(&format!("/api/v1/sources/{}/comments", source_name.0))
