@@ -4,11 +4,15 @@ use std::str::FromStr;
 
 use crate::{
     error::{Error, Result},
-    resources::{comment::EntityName, source::Id as SourceId, user::Username},
+    resources::{
+        entity_def::{EntityDef, NewEntityDef},
+        label_def::{LabelDef, NewLabelDef},
+        source::Id as SourceId,
+        user::Username,
+    },
 };
-use serde_json::Error as JsonError;
 
-#[derive(Debug, Clone, Deserialize, Serialize)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq)]
 pub struct Dataset {
     pub id: Id,
     pub name: Name,
@@ -22,6 +26,8 @@ pub struct Dataset {
     pub model_family: ModelFamily,
     pub source_ids: Vec<SourceId>,
     pub has_sentiment: bool,
+    pub entity_defs: Vec<EntityDef>,
+    pub label_defs: Vec<LabelDef>,
 }
 
 impl Dataset {
@@ -30,10 +36,10 @@ impl Dataset {
     }
 }
 
-#[derive(Debug, Clone, Deserialize, Serialize)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq, Hash)]
 pub struct Name(pub String);
 
-#[derive(Debug, Clone, Deserialize, Serialize)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq, Hash)]
 pub struct FullName(pub String);
 
 impl FromStr for FullName {
@@ -53,12 +59,12 @@ impl FromStr for FullName {
 #[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq, Hash)]
 pub struct Id(pub String);
 
-#[derive(Debug, Clone, Deserialize, Serialize)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq, Hash)]
 pub struct ModelFamily(pub String);
 
 // TODO(mcobzarenco)[3963]: Make `Identifier` into a trait (ensure it still implements
 // `FromStr` so we can take T: Identifier as a clap command line argument).
-#[derive(Debug, Clone, Deserialize, Serialize)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq, Hash)]
 pub enum Identifier {
     Id(Id),
     FullName(FullName),
@@ -88,34 +94,7 @@ impl FromStr for Identifier {
     }
 }
 
-#[derive(Debug, Serialize, Deserialize)]
-pub struct EntityDef {
-    name: EntityName,
-    title: String,
-    inherits_from: Vec<String>,
-    trainable: bool,
-}
-
-#[derive(Debug, Default, Serialize, Deserialize)]
-pub struct EntityDefs(Vec<EntityDef>);
-
-impl EntityDefs {
-    pub fn is_empty(&self) -> bool {
-        self.0.is_empty()
-    }
-}
-
-impl FromStr for EntityDefs {
-    type Err = Error;
-
-    fn from_str(string: &str) -> Result<Self> {
-        serde_json::from_str(string).map_err(|error: JsonError| Error::BadEntityDef {
-            entity_def: string.to_owned(),
-            source: error,
-        })
-    }
-}
-#[derive(Debug, Clone, Serialize, Default)]
+#[derive(Debug, Clone, Serialize, PartialEq, Eq)]
 pub struct NewDataset<'request> {
     pub source_ids: &'request [SourceId],
 
@@ -129,7 +108,10 @@ pub struct NewDataset<'request> {
     pub has_sentiment: Option<bool>,
 
     #[serde(skip_serializing_if = "Option::is_none")]
-    pub entity_defs: Option<&'request EntityDefs>,
+    pub entity_defs: Option<&'request [NewEntityDef]>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub label_defs: Option<&'request [NewLabelDef]>,
 
     #[serde(skip_serializing_if = "Option::is_none")]
     pub model_family: Option<&'request str>,
@@ -138,27 +120,27 @@ pub struct NewDataset<'request> {
     pub copy_annotations_from: Option<&'request str>,
 }
 
-#[derive(Debug, Clone, Serialize, Default)]
+#[derive(Debug, Clone, Serialize, PartialEq, Eq)]
 pub(crate) struct CreateRequest<'request> {
     pub dataset: NewDataset<'request>,
 }
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq)]
 pub(crate) struct CreateResponse {
     pub dataset: Dataset,
 }
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq)]
 pub(crate) struct GetAvailableResponse {
     pub datasets: Vec<Dataset>,
 }
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq)]
 pub(crate) struct GetResponse {
     pub dataset: Dataset,
 }
 
-#[derive(Debug, Clone, Serialize, Default)]
+#[derive(Debug, Clone, Serialize, PartialEq, Eq)]
 pub struct UpdateDataset<'request> {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub source_ids: Option<&'request [SourceId]>,
@@ -170,12 +152,12 @@ pub struct UpdateDataset<'request> {
     pub description: Option<&'request str>,
 }
 
-#[derive(Debug, Clone, Serialize, Default)]
+#[derive(Debug, Clone, Serialize, PartialEq, Eq)]
 pub(crate) struct UpdateRequest<'request> {
     pub dataset: UpdateDataset<'request>,
 }
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq)]
 pub(crate) struct UpdateResponse {
     pub dataset: Dataset,
 }
